package matrices;

import java.lang.reflect.InvocationTargetException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

public class DerbyUtil {
    private DerbyUtil() {
    }

    /**
     * Startuje DerbyEngine. Jeśli załadowano Embedded Driver można
     * od razu połączyć się z bazą danych. Jeśli załadowano ClientDriver
     * aby połączyć się z bazą danych należy wcześniej uruchomić NetworkServer
     * na tym lub zdalnym komputerze (w zależności gdzie znajduje się baza
     * danych). Jeśli baza danych znajduje się na innym komputerze to
     * połączenie z nią jest możliwe o ile na komputerze z bazą
     * zostaną ustawione odpowiednie prawa dostępu. Jeżeli nie zostaną
     * ustawione możliwe jest połączenie tylko z tego komputera, na którym
     * baza się znajduje
     *
     * @param driverName - nazwa sterownika
     * @return - true -jeśli sterownik został załadowany i silnik bazy danych
     * Derby został uruchomiony, false - jeśli sterownik bazy danych nie został
     * załadowany i silnik bazy danych nie został uruchomiony
     */
    public static boolean startDerbyEngine(String driverName) {
        return loadDBDriver(driverName);
    }

    /**
     * Ładuje wskazany sterownik bazy danych
     *
     * @param driverName - nazwa sterownika do załadowania
     * @return - true - jeśli sterownik został załadowany, false - w
     * przeciwnym przypadku
     * @throws IllegalArgumentException - wyrzucany wyjątek
     */
    private static boolean loadDBDriver(String driverName)
            throws IllegalArgumentException {
        if (driverName.equals("EmbeddedDriver")
                || driverName.equals("ClientDriver")) {
            try {
                try {
                    Class.forName("org.apache.derby.jdbc." + driverName).getDeclaredConstructors()[0].newInstance();
                } catch (InstantiationException | IllegalAccessException | InvocationTargetException e) {
                    e.printStackTrace();
                }
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
                return false;
            }
        } else {
            throw new IllegalArgumentException("Niewłaściwa nazwa sterownika");
        }
        return true;
    }

    /**
     * Tworzy połączenie z wbudowaną bazą danych. Ustawia właściwości określone
     * w łańcuchu znaków 'properties'
     *
     * @param dbName     - nazwa bazy danych lub ścieżka do bazy danych z nazwą
     *                   bazy danych na końcu
     * @param properties - pary właściwości połączenia (nazwa=właściwość)
     *                   oddzielone średnikami. Średnik na początku i końcu nie są potrzebne.
     * @return - połączenie z bazą danych
     */
    public static Connection connectEmbeddedDB(String dbName, String properties) {
        Connection conn = null;
        try {
            conn = DriverManager.getConnection("jdbc:derby:" + dbName + ";" + properties);
        } catch (Throwable e1) {
            e1.printStackTrace();
        }
        return conn;
    }

    /**
     * Zamyka polecenie
     *
     * @param stat - polecenie do zamknięcia
     */
    public static void close(Statement stat) {
        if (stat != null) {
            try {
                stat.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Zamyka połączenie
     *
     * @param conn - połączenie do zamknięcia
     */
    public static void close(Connection conn) {
        if (conn != null) {
            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Zatrzymuje silnik bazy danych Derby i wszystkie bazy danych,
     *
     * @return - true - jeśli silnik został zatrzymany i wszystkie bazy zostały
     * zamknięte, false - w przeciwnym przypadku.
     */
    public static boolean shutdownDerbyEngine() {
        boolean gotSQLExc = false;
        try {
            DriverManager.getConnection("jdbc:derby:;shutdown=true");
        } catch (SQLException se) {
            if (se.getSQLState().equals("XJ015")) {
                gotSQLExc = true;
            }
        }
        return gotSQLExc;
    }
}
